part = "both";

// Size of your printer's nozzle in mm
nozzle_size = 0.4;

// Number of walls the print should have
number_of_walls = 4;
wall_thickness = nozzle_size * number_of_walls;

// Tolerance (use 0.2 for FDM)
tolerance = 0.2; // [0.1:0.1:0.4]

// PCB x dimension in mm
x_pcb = 51.44 + tolerance;

// PCB y dimension in mm
y_pcb = 40.46 + tolerance;

// Outer dimensions in mm
x_outer = x_pcb + wall_thickness * 2;
y_outer = y_pcb + wall_thickness * 2;
z_lower = 9;
z_upper = 9;

lip_height = 1;

// Radius for rounded corners in mm
radius = 3; // [1:20]

// Writing
letter_height = 0.5;
letter_size = 12;

// PCB holes
hole_spacing_x = 44.64;
hole_spacing_y = 33.4;

hole11 = [- hole_spacing_x / 2, - hole_spacing_y / 2, 0];
hole12 = [+ hole_spacing_x / 2, - hole_spacing_y / 2, 0];
hole21 = [- hole_spacing_x / 2, + hole_spacing_y / 2, 0];
hole22 = [+ hole_spacing_x / 2, + hole_spacing_y / 2, 0];

// PCB support columns
support_height = 3;
support_dia = 6.8;

// Screw parameters
screw_dia = 3;
screw_head_dia = 5.2;
screw_head_height = 2.4;
hollow_height = z_upper + z_lower - support_height - 1.6 - 0.4 - wall_thickness * 2;

$fn = 100;


module jack(top = false)
{
    jack_dia = 6.6;
    
    rotate(a = [90, 0, 0])
    linear_extrude(10) {
         circle(d = jack_dia);
    }
    if (top) {
        // So the lip doesn't get an overhang
        translate([0, -jack_dia, -jack_dia / 2]) cube(jack_dia, center = true);
    }
}


module cutouts(top = 0)
{
    // 3.5mm jacks
    translate([1.9, -y_pcb / 2 + 5, 9.2]) jack(top);
    translate([-11.8, -y_pcb / 2 + 5, 9.2]) jack(top);
    
    translate([-x_pcb / 2 - 2.5, 18.6, 5])
    rotate(a = [90, 0, 90])
    linear_extrude(5) {
        // USB
        translate([-18, support_height + 3, 0])
        minkowski() {
            square([7, 3]);
            circle(1);
        }
    }
}


// PCB support structure
module support()
{
    linear_extrude(support_height) circle(d = support_dia);
}


// Hole for the screw
module screw_hole()
{
    linear_extrude(support_height + wall_thickness + tolerance) circle(d = screw_dia + tolerance);
    linear_extrude(screw_head_height + tolerance) circle(d = screw_head_dia + tolerance);
}


// Plastic to screw into
module screw_nut()
{
    linear_extrude(hollow_height) difference() {
        circle(d = support_dia);
        circle(d = screw_dia - tolerance);
    };
}


module bottom_box()
{
    difference() {
        union() {
            difference() {
                // Solid box
                linear_extrude(z_lower) {
                    minkowski() {
                        square([x_outer - radius * 2, y_outer - radius * 2], center = true);
                        circle(radius);
                    }
                }
                
                // Hollow out
                translate([0, 0, wall_thickness]) linear_extrude(z_lower) {
                    minkowski() {
                        square([x_outer - radius * 2, y_outer - radius * 2], center = true);
                        circle(radius - wall_thickness);
                    }
                }

                // Lip
                translate([0, 0, z_lower - 1]) linear_extrude(1.0) {
                    minkowski() {
                        square([x_outer - radius * 2, y_outer - radius * 2], center = true);
                        circle(radius - wall_thickness / 2);
                    }
                }        
            }

            // Place supports in box
            translate([0, 0, wall_thickness]) {
                translate(hole11) support();
                translate(hole12) support();
                translate(hole21) support();
                translate(hole22) support();    
            }    
        }
        cutouts();
        
        // Remove screw holes
        translate(hole11) screw_hole();
        translate(hole12) screw_hole();
        translate(hole21) screw_hole();
        translate(hole22) screw_hole();
    }
}


module top_box()
{
    difference() {
        union() {
            difference() {
                union() {
                    // Solid box
                    linear_extrude(z_upper) {
                        minkowski() {
                            square([x_outer - radius * 2, y_outer - radius * 2], center = true);
                            circle(radius);
                        }
                    }

                    // Add lip
                    linear_extrude(z_upper + 1) {
                        minkowski() {
                            square([x_outer - radius * 2, y_outer - radius * 2], center = true);
                            circle(radius - wall_thickness / 2 - tolerance);
                        }
                    }
                }
                    
                // Hollow out
                translate([0, 0, wall_thickness]) linear_extrude(z_upper + 1) {
                    minkowski() {
                        square([x_outer - radius * 2, y_outer - radius * 2], center = true);
                        circle(radius - wall_thickness);
                    }
                }
            }

            // Add hollow pins            
            translate([0, 0, wall_thickness]) {
                translate(hole11) screw_nut();
                translate(hole12) screw_nut();
                translate(hole21) screw_nut();
                translate(hole22) screw_nut();
            }    
        }

        // Recycle cutouts from bottom box
        translate([0, 0, z_upper + z_lower]) rotate(a = [0, 180, 0]) cutouts(top = true);
        
        // Carve text
        translate([0, 0, -0.1]) linear_extrude(height = letter_height + 0.1) {
            rotate([0, 180, 180]) text("QLUB", size = letter_size, font = "Saxony\\-Serial:style=Regular", halign = "center", valign = "center", $fn = 16);
        }
    }       
}


module both()
{    
	bottom_box();
    translate([0, y_outer + wall_thickness, 0]) top_box();
//    translate([0, 0, support_height + 1.6]) color([0, 0.5, 0, 1.0]) import("QLUB_pcb.stl");
//    #translate([0, 0, z_lower + z_upper]) rotate([0, 180, 0]) top_box();    
}


module print_part()
{
	if (part == "bottom") {
		bottom_box();
	} else if (part == "top") {
		top_box();
	} else if (part == "both") {
		both();
	} else {
		both();
	}
}

print_part();
